#!/usr/bin/env python
# coding: utf-8

import threading
import struct

# Process one client request: Read, Price, Write, Close
class ClientRequest(threading.Thread):
    def __init__(self, conn, address, processFunc):
        threading.Thread.__init__(self)
        self.conn = conn
        self.address = address
        self.processFunc = processFunc

    # Vectalis-pricer-message have message lenth as the first 4 bytes of the tcp-stream.
    def run(self):
        try:
            print ("Begin client request from " + str(self.address))
            incomingMessage = self.__receiveMessage()
            try:
                outcomingMessage = self.processFunc(incomingMessage)
            except Exception as ex:
                self.__sendMessage("Python error: " + repr(ex))
                raise
            self.__sendMessage(outcomingMessage)
            print ("End client request from " + str(self.address))
        finally:
            self.conn.close()

    def __receiveMessage(self):
        messageSizeBytes = self.conn.recv(4) # 4 = length of int32 
        if len(messageSizeBytes) == 0:#Connection already closed
            return ""
        messageSizeInt = struct.unpack("<I", messageSizeBytes)[0]
        incomingMessageBytes = b''
        missingSize = messageSizeInt - len(incomingMessageBytes)
        while missingSize > 0:#For big messages
            chunk = self.conn.recv(missingSize)
            if chunk == '':
                #socket connection broken
                return ""
            incomingMessageBytes = incomingMessageBytes + chunk
            missingSize = messageSizeInt - len(incomingMessageBytes)
        #print "Received Bytes size: " + str(len(incomingMessageBytes))
        incomingMessage = incomingMessageBytes.decode("utf-8")
        if len(incomingMessage) < 100:
            print (" Received message: " + str(incomingMessage))
        else:
            print (" Received message: " + str(incomingMessage[0:90]) + "...")
        return incomingMessage
    
    def __sendMessage(self, outcomingMessage):
        if len(outcomingMessage) < 50:
            print (" Send message: " + str(outcomingMessage))
        else:
            print (" Send message: " + str(outcomingMessage[0:40]) + "...")
        outcomingMessageBytes = outcomingMessage.encode("utf-8")
        outcomingMessageSizeInt = len(outcomingMessageBytes)
        outcomingMessageSizeBytes = struct.pack("<I", outcomingMessageSizeInt) 
        try:
            #print "Send size: " + str(outcomingMessageSizeInt)
            self.conn.sendall(outcomingMessageSizeBytes)
            #print "Send message: " + outcomingMessage
            self.conn.sendall(outcomingMessageBytes)
        except OSError as err: 
            
            # Do not log error when there is nothing to send (happen on when UTS start the connection)
             #10053 = 'Software caused connection abort'
            if not(err.errno == 10053 and outcomingMessage == ""):
                raise 

