#!/usr/bin/env python
# coding: utf-8

import xml.etree.ElementTree as xml
from VectalisCore.DerivativeData import *
from VectalisCore.Distinctor import Distinctor

class XmlSerializer:
    utsPricerMessageVersion = "4.1"
    vectalisCoreVersion = "7.9.0.20513"

    '''
     Read the QuotesEvaluationRequest XML 4.1 into a business object of type DerivativeData
     The XML tree is:
    
      XML
      |-QuotesEvaluationRequest
        |-Quote
          |-Derivative
          |-Legs
          | |-Leg
          |   |-LegDerivativeType
          |   |-UnderlyingDefinition
          |   | |-UnderlyingConstituent
          |   |   |-Underlying
          |   |    |-UnderlyingReference 
          |   |-LegItems
          |-TradingInterests
          |-SalesTradeInfo
            Custom trade info
    '''
    def createDerivativeDataFromXml(self, xTree): 
        derivativeData = DerivativeData()
        distinctor = Distinctor()
        xQuotesEvaluationRequest = xTree.find("./QuotesEvaluationRequest")
        if xQuotesEvaluationRequest is None:
            raise "Invalid " + XmlSerializer.utsPricerMessageVersion + " pricing message"

        #derivativeData.request
        request = Request()
        request.xmlPricingMessageVersion = xQuotesEvaluationRequest.attrib[ "XmlPricingMessageVersion"]
        if request.xmlPricingMessageVersion != XmlSerializer.utsPricerMessageVersion:
            raise "XmlPricingMessageVersion is '" + request.xmlPricingMessageVersion + "' while '" + XmlSerializer.utsPricerMessageVersion + "' is expected."
        request.requesterEntityCode = xQuotesEvaluationRequest.attrib["RequesterEntityCode"]
        request.requesterCode = xQuotesEvaluationRequest.attrib["RequesterCode"]
        request.requesterAddresseeCode = xQuotesEvaluationRequest.attrib[ "RequesterAddresseeCode"]
        request.windowsUsername = xQuotesEvaluationRequest.attrib["WindowsUsername"]
        request.operationDate = xQuotesEvaluationRequest.attrib["OperationDate"]
        request.pricingParameters = xQuotesEvaluationRequest.attrib["ParameterPricingService"]
        derivativeData.request = request

        #Quote
        for xQuote in xQuotesEvaluationRequest.findall("./Quote"):
            quote = Quote()
            derivativeData.quotes.append(quote)
            quote.quoteMessageId = xQuote.attrib["QuoteMessageId"]
            quote.quoteId = xQuote.attrib["QuoteId"]
            quote.listedOrOtc = xQuote.attrib["ListedOrOtc"]
            quote.quotationType = xQuote.attrib["QuotationType"]
            quote.deferredPremium = xQuote.attrib["DeferredPremium"]
            quote.providerDisplayCode = xQuote.attrib["ProviderDisplayCode"]
            quote.providerEntityCode = xQuote.attrib["ProviderEntityCode"]
            quote.providerCode = xQuote.attrib["ProviderCode"]
            quote.providerEntityName = xQuote.attrib["ProviderEntityName"]
            quote.providerName = xQuote.attrib["ProviderName"]
            quote.providerComment = xQuote.attrib["ProviderComment"]
            quote.overwriteComment = xQuote.attrib["OverwriteComment"]
            quote.packageId = xQuote.attrib["PackageId"]
            quote.packageMultiplier = xQuote.attrib["PackageMultiplier"]
            quote.isPackageChoice = xQuote.attrib["IsPackageChoice"]

            xDerivativeType = xQuote.find("./DerivativeType")
            quote.derivativeTypeCode = xDerivativeType.attrib["Code"]
            quote.derivativeTypePricerCode = xDerivativeType.attrib["PricerCode"]
            quote.derivativeTypeName = xDerivativeType.attrib["Name"]
            
            xLegs = xQuote.find("./Legs")
            #Leg
            for xLeg in xLegs.findall("./Leg"):
                legConstituent = LegConstituent()
                leg = Leg()
                legConstituent.index = xLeg.attrib["Index"]
                legConstituent.multiplier = xLeg.attrib["Multiplier"]
                quote.legConstituents.append(legConstituent)
                xLegDerivativeType = xLeg.find("./LegDerivativeType")
                leg.basketType = xLegDerivativeType.attrib["BasketType"]
                leg.derivativeTypeCode = xLegDerivativeType.attrib["Code"]
                leg.isPriceInPercent = xLegDerivativeType.attrib["IsPriceInPercent"]
                
                #Underlying
                xUnderlyingDefinition = xLeg.find("./UnderlyingDefinition")
                underlyingDefinition = UnderlyingDefinition()
                underlyingDefinition.name = xUnderlyingDefinition.attrib["Name"]
                underlyingDefinition.displayName = xUnderlyingDefinition.attrib["DisplayName"]
                
                #UL Constituent
                for xUnderlyingConstituent in xUnderlyingDefinition.findall("./UnderlyingConstituent"):
                    xUnderlying = xUnderlyingConstituent.find("./Underlying")
                    xUnderlyingReference = xUnderlying.find("./UnderlyingReference")
                    underlyingReference = UnderlyingReference()
                    underlying = Underlying()
                    underlyingConstituent = UnderlyingConstituent() # Peut etre a completer
                    
                    underlyingConstituent.weight = xUnderlyingConstituent.attrib["Weight"]
                    underlying.vectalisCode = xUnderlying.attrib["Code"]
                    underlying.pricerCode = xUnderlying.attrib["PricerCode"]
                    underlying.typeCode = xUnderlying.attrib["TypeCode"]
                    underlying.currencyCode = xUnderlying.attrib["CurrencyCode"]
                    underlying.displayCode = xUnderlying.attrib["DisplayCode"]
                    underlying.reutersCode = xUnderlying.attrib["Reuters"]
                    underlying.bloombergCode = xUnderlying.attrib["Bloomberg"]
                    underlyingReference.providedReferenceSpot = xUnderlyingReference.attrib["ReferenceSpot"]
                    underlyingReference.providedFutureExpiryDate = xUnderlyingReference.attrib["FutureExpiryDate"]
                    underlyingReference.providedBasis = xUnderlyingReference.attrib["FutureBasis"]
                    underlyingReference.providedDelta = xUnderlyingReference.attrib["Delta"]   
                    
                    underlyingReference.underlying = distinctor.internUnderlying(underlying)
                    underlyingConstituent.underlyingReference = distinctor.internUnderlyingReference(underlyingReference)
                    underlyingDefinition.constituents.append(underlyingConstituent)
                leg.underlyingDefinition = distinctor.internUnderlyingDefinition(underlyingDefinition)
                
                #Leg Items
                xLegItems = xLeg.find("./LegItems")
                for xLegItem in xLegItems.findall("./LegItem"):
                    legItemName = xLegItem.attrib["Name"]
                    legItemValue = xLegItem.attrib["Value"]
                    leg.legItems[legItemName] = legItemValue
                legConstituent.leg = leg
            
            #Quote.TradingInterest
            xTradingInterests = xQuote.find("./TradingInterests")
            for xTradingInterest in xTradingInterests.findall("./TradingInterest"):
                tradingInterest = TradingInterest()
                tradingInterest.code = xTradingInterest.attrib["Code"]
                tradingInterest.price = xTradingInterest.attrib["Price"]
                tradingInterest.size = xTradingInterest.attrib["Size"]
                tradingInterest.time = xTradingInterest.attrib["Time"]
                tradingInterest.conditionString = xTradingInterest.attrib["ConditionString"]
                tradingInterest.comment = xTradingInterest.attrib["Comment"]
                quote.tradingInterests.append(tradingInterest)
            
            #Quote.SalesTradeInfo
            xSalesTradeInfo = xQuote.find("./SalesTradeInfo")
            salesTradeInfo = SalesTradeInfo()
            salesTradeInfo.salesOtcSize = xSalesTradeInfo.attrib["SalesOtcSize"]
            salesTradeInfo.salesListedSize = xSalesTradeInfo.attrib["SalesListedSize"]
            salesTradeInfo.salesPandL = xSalesTradeInfo.attrib["SalesPandL"]
            salesTradeInfo.salesComment = xSalesTradeInfo.attrib["SalesComment"]
            salesTradeInfo.salesCustomString1 = xSalesTradeInfo.attrib["SalesCustomString1"]
            salesTradeInfo.salesCustomString2 = xSalesTradeInfo.attrib["SalesCustomString2"]
            salesTradeInfo.salesCustomString3 = xSalesTradeInfo.attrib["SalesCustomString3"]
            salesTradeInfo.salesCustomString4 = xSalesTradeInfo.attrib["SalesCustomString4"]
            salesTradeInfo.salesCustomString5 = xSalesTradeInfo.attrib["SalesCustomString5"]
            salesTradeInfo.salesCustomString6 = xSalesTradeInfo.attrib["SalesCustomString6"]
            salesTradeInfo.salesCustomDouble1 = xSalesTradeInfo.attrib["SalesCustomDouble1"]
            salesTradeInfo.salesCustomDouble2 = xSalesTradeInfo.attrib["SalesCustomDouble2"]
            salesTradeInfo.salesCustomBool1 = xSalesTradeInfo.attrib["SalesCustomBool1"]
            salesTradeInfo.salesCustomBool2 = xSalesTradeInfo.attrib["SalesCustomBool2"]
            quote.salesTradeInfo = salesTradeInfo
            
            #Quote.CustomtradeInfo
            xCustomTradeInfo = xQuote.find("./CustomTradeInfo")
            customTradeInfo = CustomTradeInfo()
            customTradeInfo.customString1 = xCustomTradeInfo.attrib["CustomString1"]
            customTradeInfo.customString2 = xCustomTradeInfo.attrib["CustomString2"]
            customTradeInfo.customString3 = xCustomTradeInfo.attrib["CustomString3"]
            customTradeInfo.customString4 = xCustomTradeInfo.attrib["CustomString4"]
            customTradeInfo.customString5 = xCustomTradeInfo.attrib["CustomString5"]
            customTradeInfo.customDouble1 = xCustomTradeInfo.attrib["CustomDouble1"]
            customTradeInfo.customDouble2 = xCustomTradeInfo.attrib["CustomDouble2"]
            customTradeInfo.customDouble3 = xCustomTradeInfo.attrib["CustomDouble3"]
            customTradeInfo.customDouble4 = xCustomTradeInfo.attrib["CustomDouble4"]
            customTradeInfo.customDouble5 = xCustomTradeInfo.attrib["CustomDouble5"]
            customTradeInfo.customBool1 = xCustomTradeInfo.attrib["CustomBool1"]
            customTradeInfo.customBool2 = xCustomTradeInfo.attrib["CustomBool2"]
            customTradeInfo.customBool3 = xCustomTradeInfo.attrib["CustomBool3"]
            customTradeInfo.customBool4 = xCustomTradeInfo.attrib["CustomBool4"]
            customTradeInfo.customBool5 = xCustomTradeInfo.attrib["CustomBool5"]
            quote.customTradeInfo = customTradeInfo
            
        #End XML
        
        #Finalize derivativeData
        for quote in derivativeData.quotes:
            # init underlyingDefinition.hasWeight
            for legConstituent in quote.legConstituents:
                leg = legConstituent.leg
                leg.underlyingDefinition.hasWeight = any(c.weight != "" for c in leg.underlyingDefinition.constituents)
            # init quote.singleUnderlyingReference
            if len(quote.legConstituents) >= 1 and all(len(legConstituent.leg.underlyingDefinition.constituents) == 1 for legConstituent in quote.legConstituents):
                firstUnderlyingReference = quote.legConstituents[0].leg.underlyingDefinition.constituents[0].underlyingReference
                if all(legConstituent.leg.underlyingDefinition.constituents[0].underlyingReference == firstUnderlyingReference for legConstituent in quote.legConstituents):
                    quote.singleUnderlyingReference = firstUnderlyingReference

        derivativeData.underlyingDefinitions.extend(distinctor.distinctUnderlyingDefinitions.values())
        derivativeData.underlyingReferences.extend(distinctor.distinctUnderlyingReferences.values())
        derivativeData.underlyings.extend(distinctor.distinctUnderlyings.values())
        return derivativeData

    def __createXmlPricingReply_Error(self, xQuote, quote):
        qError = quote.getError()
        if qError is not None:
            xError = xml.Element("Error")
            xQuote.append(xError)
            xError.text = "Python: " + repr(qError)
    def __createXmlPricingReply_ReferenceItem(self, xQuote, quote):
        if quote.singleUnderlyingReference is None:
            #we do not support multi-underlyings
            return

        xRefItems = xml.Element("ReferenceItems")
        xQuote.append(xRefItems)

        for (name, value) in quote.returnedReferenceItems.items():
            xReferenceItem = xml.Element("ReferenceItem")
            xRefItems.append(xReferenceItem)
            xReferenceItem.attrib["Name"] = name
            if isinstance(value, Parameter):    
                xReferenceItem.attrib["Value"] = str(value.value)
            else:
                xReferenceItem.attrib["Value"] = str(value)
    def __createXmlPricingReply_LegItem(self, xLeg, name, legItem):
        xLegItem = xml.Element("LegItem")
        xLeg.append(xLegItem)
        xLegItem.attrib["Name"] = name

        if isinstance(legItem, Parameter):
            xLegItem.attrib["Value"] = str(legItem.value)
            if legItem.header is not None:
                xLegItem.attrib["Header"] = str(legItem.header)
            if legItem.displayFormat is not None:
                xLegItem.attrib["DisplayFormat"] = str(legItem.displayFormat)
            if legItem.foreColor is not None:
                xLegItem.attrib["ForeColor"] = str(legItem.foreColor)
            if legItem.backColor is not None:
                xLegItem.attrib["BackColor"] = str(legItem.backColor)
        else:
            xLegItem.attrib["Value"] = str(legItem)   
    def __createXmlPricingReply_TradingInterestItem(self, xTradingInterest, name, interestItem):
        xTradingInterestItem = xml.Element("TradingInterestItem")
        xTradingInterest.append(xTradingInterestItem)
        xTradingInterestItem.attrib["Name"] = name

        if isinstance(interestItem, Parameter):
            xTradingInterestItem.attrib["Value"] = str(interestItem.value)
            if interestItem.header is not None:
                xTradingInterestItem.attrib["Header"] = str(interestItem.header)
            if interestItem.displayFormat is not None:
                xTradingInterestItem.attrib["DisplayFormat"] = str(interestItem.displayFormat)
            if interestItem.foreColor is not None:
                xTradingInterestItem.attrib["ForeColor"] = str(interestItem.foreColor)
            if interestItem.backColor is not None:
                xTradingInterestItem.attrib["BackColor"] = str(interestItem.backColor)
        else:
            xTradingInterestItem.attrib["Value"] = str(interestItem)  
    def createXmlPricingReply(self, derivativeData):
        xResponse = xml.Element("XML")
        xQuotesEvaluationReply = xml.Element("QuotesEvaluationReply")
        xResponse.append(xQuotesEvaluationReply)
        for quote in derivativeData.quotes:
            xQuote = xml.Element("Quote")
            xQuotesEvaluationReply.append(xQuote)
            xQuote.attrib["QuoteId"] = quote.quoteId
            xQuote.attrib["QuoteMessageId"] = quote.quoteMessageId

            self.__createXmlPricingReply_Error(xQuote, quote)
            self.__createXmlPricingReply_ReferenceItem(xQuote, quote)

            xLegs = xml.Element("Legs")
            xQuote.append(xLegs)
            for legConstituent in quote.legConstituents:
                xLeg = xml.Element("Leg")
                xLegs.append(xLeg)
                self.__createXmlPricingReply_LegItem(xLeg, "Index", legConstituent.index)
                for greekName in legConstituent.leg.returnedGreeks.keys(): 
                    self.__createXmlPricingReply_LegItem(xLeg, greekName, legConstituent.leg.returnedGreeks[greekName])
            
            xLeg0 = xml.Element("Leg")
            xLegs.append(xLeg0)
            self.__createXmlPricingReply_LegItem(xLeg0, "Index", 0)
            for greekName in quote.returnedGreeks.keys():                 
                self.__createXmlPricingReply_LegItem(xLeg0, greekName, quote.returnedGreeks[greekName])

            xTradingInterests = xml.Element("TradingInterests")
            xQuote.append(xTradingInterests)
            for tradingInterest in quote.tradingInterests:
                xTradingInterest = xml.Element("TradingInterest")
                xTradingInterests.append(xTradingInterest)
                self.__createXmlPricingReply_TradingInterestItem(xTradingInterest, "Name", tradingInterest.code)
                self.__createXmlPricingReply_TradingInterestItem(xTradingInterest, "Price", tradingInterest.price)
                for greekName in tradingInterest.returnedGreeks.keys(): 
                    self.__createXmlPricingReply_TradingInterestItem(xTradingInterest, greekName, tradingInterest.returnedGreeks[greekName])
        return xml.tostring(xResponse, encoding='unicode')

    def createXmlCommandListReply(self, commandList):
        xResponse = xml.Element("XML")
        xCommandList = xml.Element("Commands")
        xResponse.append(xCommandList)
        for command in commandList:
            xCommand = xml.Element("Command")
            xCommandList.append(xCommand)
            xCommand.attrib["Name"] = command
            
        return xml.tostring(xResponse, encoding='unicode')

    def createXmlCustomCommandFromXml(self, xTree):
        customCommand = CustomCommandRequest()
        #<XML><CustomCommand><Name/><Request/></CustomCommand></XML>
        xCustomCommand = xTree.find("./CustomCommand")
        if xCustomCommand is None:
            raise "Invalid custom command message."

        #command name
        xCommandName = xCustomCommand.find("./Name")
        if xCommandName is None:
            raise "Invalid custom command name."
        customCommand.command = xCommandName.text

        #command request
        xRequest = xCustomCommand.find("./Request")
        if xRequest is None:
            raise "Invalid custom command request."

        xRequestTree = xml.fromstring(xRequest.text)
        if len(xRequestTree.findall("./QuotesEvaluationRequest")) <= 0: # CustomCommand request
            raise "Custom command request is not valid."
        customCommand.derivativeData = self.createDerivativeDataFromXml(xRequestTree)       
        
        return customCommand
        





