
#!/usr/bin/env python
# coding: utf-8


import xml.etree.ElementTree as xml
from VectalisCore.Communication.TcpServer import TcpServer
from VectalisCore.XmlSerializer import XmlSerializer
from BankImplementation.PriceQuotes import evaluateQuotes
from BankImplementation.PricingCommands import getPricingCommands, processPricingCommand


def processRequest(inputXml):
    if inputXml == "":
        print (" Empty request")
        return "" # Assume it is a test connection

    xTree = xml.fromstring(inputXml)
    serializer = XmlSerializer()
    if len(xTree.findall("./QuotesEvaluationRequest")) > 0: # Pricing request
        derivativeData = serializer.createDerivativeDataFromXml(xTree)       
        print (" QuotesEvaluationRequest on quote(s): " + ", ".join(str(quote.quoteId) for quote in derivativeData.quotes))
        evaluateQuotes(derivativeData)
        outputXml = serializer.createXmlPricingReply(derivativeData)
        return outputXml
    if len(xTree.findall("./CommandListRequest")) > 0: # CommandList Request
        print (" CommandList Request")
        commands = getPricingCommands()
        outputXml = serializer.createXmlCommandListReply(commands)
        return outputXml
    if len(xTree.findall("./CustomCommand")) > 0: # CommandList Request
        customCommand = serializer.createXmlCustomCommandFromXml(xTree)
        print (" Custom command '" + customCommand.command + "' on quote(s):" +", ".join(str(quote.quoteId) for quote in customCommand.derivativeData.quotes))
        hasRecomputedDerivative = processPricingCommand(customCommand)
        if hasRecomputedDerivative:
            outputXml = serializer.createXmlPricingReply(customCommand.derivativeData)
        else:
            outputXml = ""
        return outputXml
    print (" Unknow command: " + inputXml)
    return "" # Not implemented, TODO


if __name__ == "__main__":
    print ("Starting python pricer for UTS")
    print ("VectalisCore Version: " + XmlSerializer.vectalisCoreVersion)
    s = TcpServer(processRequest)
    s.run()
 

